package fr.hugol;

import org.graphstream.algorithm.Toolkit;
import org.graphstream.algorithm.generator.BarabasiAlbertGenerator;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.BreadthFirstIterator;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSource;
import org.graphstream.stream.file.FileSourceEdge;

import javax.tools.Tool;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Locale;

public class Main {
    public static void main(String[] args) {
        System.setProperty("org.graphstream.ui", "swing");
        var g = createRandom();

        var nodeCount = g.getNodeCount();
        var edgeCount = g.getEdgeCount();
        var avgDegree = Toolkit.averageDegree(g);
        var avgClusteringCoeff = Toolkit.averageClusteringCoefficient(g);

        System.out.println("\nQ1");
        System.out.println("- Nodes: " + nodeCount);
        System.out.println("- Edges: " + edgeCount);
        System.out.println("- Avg. degree: " + avgDegree);
        System.out.println("- Avg. clustering coeff.: " + avgClusteringCoeff);

        System.out.println("\nQ2");
        System.out.println("- Avg. clustering coeff. for random graph: " + (avgDegree / nodeCount));

        System.out.println("\nQ3");
        var avgDegreeForConnectedRegime = Math.log(nodeCount);
        System.out.println("- Is connected: " + Toolkit.isConnected(g));
        System.out.println("- All random graphs are connected: " + (avgDegree > avgDegreeForConnectedRegime));
        System.out.println("- Avg. degree for connected regime: " + avgDegreeForConnectedRegime);

        System.out.println("\nQ4");
        System.out.println("Writing degree distribution file...");
        try {
            generateDegreeDistributionFile(g);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        System.out.println("Done.");

        System.out.println("\nQ5");
        //System.out.println("- Avg. distance: " + getAverageDistance(g, 1000));

        System.out.println("\nQ6");
        System.out.println("Writing distance distribution file...");
        /*
        try {
            generateDistanceDistributionFile(g);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

         */
        System.out.println("Done.");
    }

    public static Graph create() {
        var g = new SingleGraph("TP");

        FileSource fs = new FileSourceEdge();
        fs.addSink(g);
        try {
            fs.readAll("src/main/resources/data/com-dblp.ungraph.txt");
        } catch (IOException ignored) {
        } finally {
            fs.removeSink(g);
        }

        return g;
    }

    public static Graph createRandom() {
        Graph g = new SingleGraph("Random");
        Generator gen = new RandomGenerator(6.75);
        gen.addSink(g);
        gen.begin();
        for (int i = 0; i < 100-5; i++)
            gen.nextEvents();
        gen.end();
        return g;
    }


    public static Graph createBarabasiAlbert() {
        Graph g = new SingleGraph("Barabàsi-Albert");
        Generator gen = new BarabasiAlbertGenerator(6);

        gen.addSink(g);
        gen.begin();

        for (int i = 0; i < 100; i++) {
            gen.nextEvents();
        }

        gen.end();
        return g;
    }

    public static void generateDegreeDistributionFile(Graph graph) throws IOException {
        FileWriter fileWriter = new FileWriter("gnuplot/dd_dblp.dat");
        PrintWriter printWriter = new PrintWriter(fileWriter);

        int[] dd = Toolkit.degreeDistribution(graph);
        for (int k = 0; k < dd.length; k++) {
            if (dd[k] != 0) {
                printWriter.printf(Locale.US, "%6d%20.8f%n", k, (double) dd[k] / graph.getNodeCount());
            }
        }
        printWriter.close();
    }

    public static void generateDistanceDistributionFile(Graph graph) throws IOException {
        FileWriter fileWriter = new FileWriter("gnuplot/distance_distribution.dat");
        PrintWriter printWriter = new PrintWriter(fileWriter);

        // Assuming you have a method to calculate all pairs shortest path
        int sampleSize = 1000;
        int[] distanceDistribution = calculateDistanceDistribution(graph, sampleSize);
        int nodeCount = graph.getNodeCount();
        for (int k = 0; k < distanceDistribution.length; k++) {
            if (distanceDistribution[k] != 0) {
                printWriter.printf(Locale.US, "%6d%20.8f%n", k, (double)distanceDistribution[k] / (sampleSize * nodeCount -1));
            }
        }
        printWriter.close();
    }


    public static int[] calculateDistanceDistribution(Graph graph, int sampleSize) {
        int[] distanceDistribution = new int[sampleSize];
        for (Node node : Toolkit.randomNodeSet(graph, sampleSize)) {
            BreadthFirstIterator iterator = new BreadthFirstIterator(node);
            while (iterator.hasNext()) {
                Node currentNode = iterator.next();
                int distance = iterator.getDepthOf(currentNode);
                if (distance < sampleSize) {
                    distanceDistribution[distance]++;
                }
            }
        }
        return distanceDistribution;
    }

    /**
     * Calcule la distance moyenne d'un Graphe à l'aide du parcours en largeur
     */
    public static double getAverageDistance(Graph g, int nodeCount) {
        long sum = 0;
        for (var node : Toolkit.randomNodeSet(g, nodeCount)) {
            var iterator = new BreadthFirstIterator(node);
            while (iterator.hasNext()) {
                sum += iterator.getDepthOf(iterator.next());
            }
        }
        return (1.0D / (nodeCount * g.getNodeCount())) * (double) sum;
    }
}
